#ifndef _LIGOCONT_H
#define _LIGOCONT_H

#ifdef __cplusplus
extern "C" {
#endif

/*----------------------------------------------------------------------*
 *                      LIGO Peak-Trough Monitor                        *
 *									*
 *	LIGOCONT.H - Main structure definitions				*
 *          Adapted from CSCONT.H, USGS                                 *  
 *									*
 *	Contains information used to analyze seismic events.  There	*
 *	is one of these buffers for each discrete component of seismic	*
 *	data that is being event detected.				*
 *----------------------------------------------------------------------*/

/*---------------------------Continuity Structure-----------------------*/

struct con_sto {

/*-----------------Variables used in e_detect---------------------------*/

	TEXT *ch_name;	/* Name of channel stored in e_detect 	*/
	LONG sam_ch;	        /* Total number samples per channel     *
				 * of the record			*/
	BYTE cur_rec;		/* Index of the current record */

/*-------------Variable Used in event subroutine-------------------------*/

	LONG buf_flg[E_B];	/* flags: 1 if >= th2; 2 if >= th1 */
	LONG buf_sc[E_B];	/* summed delta sample counts */
	LONG buf_amp[E_B];	/* P-T amplitudes values */
	LONG buf_tim[E_B];	/* time coordinate of P-T values */
	BYTE buf_rec[E_B];	/*  record number array */

	LONG abuf_sc[4];	/* last 4 delta sample counts */
	LONG abuf_amp[4];	/* last 4 P-T amplitudes */
	LONG abuf_tim[4];	/* time cord. of last 4 P-T values */
	BYTE abuf_rec[4];	/* companion to buf_rec */

	LONG last_amp;          /* last P-T value       */

	BOOL epf;		/* event possible flag */
	BOOL evon;		/* event detected flag */
	BOOL icheck;		/* flag, ensures period estimate */

	LONG filhi;	/* see OF Report 83-785 */
	LONG fillo;	/* see OF Report 83-785 */
	LONG iwin;	/* window length (samples) */
	LONG n_hits;	/* # P-T >= th2 for detection  */

	LONG fst_flg;	/* index to first flagged P-T value  */
	LONG indx;	/* saves fst_flg */
	LONG lst_flg;	/* index to last flagged P-T value */
	LONG lst_flg2;	/* lst_flg corrected for overflow */
	LONG lst_pt;	/* index to last P-T value processed */
	LONG lst_pt2;	/* lst_pt corrected for overflow */
	LONG index2;	/* counter,checked for overflow */

	BOOL iset;	/* flag, = 1 when P-T >= th1 */

	LONG jj;	/* index for abrev buffers abuf */
	LONG sumdsc;	/* sum delta sample count */
	LONG sumflg;	/* number of P-T values > th2 */
	LONG last_sc;	/* loaded to abuf_sc[] */
	LONG last_tim;	/* loaded to abuf_tim[] */
	BYTE last_rec;	/*  record of last_tim */

	LONG th1;	/* largest detection threshold */
	LONG th2;	/* smallest detection threshold */

/*---------------------Variables used in p_one----------------------------*/

	LONG last_y;	/* amplitude coordinate of previous sample */
	LONG last_x;	/* time coordinate of previous sample */
	BYTE rec_last_x;	/*  Record of last time coordinate */
	LONG sum_s_c;	/* samples from last P or T to current sample */
	LONG s_sum_sc;	/* samples between last two P-T values */
	LONG index;	/* counter, calls ptwo */
	LONG max_y;	/* amplitude coordinate of P or T value */
	LONG tim_of_max;	/* time coordinate of P or T value */
	BYTE rec_of_max;	/*  record of tim_of_max */
	BOOL prev_slope;	/* sign of last difference  */
	LONG maxamp;	/* abs max of 20 consec. P-T values < thx */
	LONG thx;	/* upper bound for noise est. */
	LONG s_amp;	/* signed amplitude of P-T value */

/*----------------------Variables used in p_two---------------------------*/

	LONG tsstak[16];	/* contains set of maxamp values */
	LONG twosd;	/* statistical dispersion of P-T values */
	LONG kk;	/* index for tsstak[] */
	LONG val_avg;	/* the number of values in tsstak[] */

/*----------------------Variables used in onsetq--------------------------*/

	LONG th3;		/* threshold for estimating onset */
	LONG def_tc;	/* time correction for onset (default) */
	BOOL ponset;		/* Print onsets? */
	VOID (*onsproc)();	/* Call subroutine upon onset */

/*----------------------Variables used in count_dn-------------------------*/

	ULONG wait_blk;	/* controls re-activation of detector and *
				 *  recording time in event coda 	*/
	LONG itc;	/* counter for interval of the event */
	LONG nn;	/* counter for itc */

/*----------------------Variables used in xth-----------------------------*/

	ULONG xth1; 	/* coded threshold factor -- xth*/
	ULONG xth2;	/*           "			*/
	ULONG xth3;	/*           "			*/
	ULONG xthx;	/*           "			*/
			          /* th1>th2>thx>th3 or th1>thx>th2>th3	*/

/*----------------------Variables to set up the detector----------------------*/

	LONG kont_per;  /* counts per period */
	LONG ns_sam;	/* nanoseconds per sample */
	LONG haf_per;	/* samples per one-half period */
	LONG sam_sec;	/* sample rate of the digital data */
	LONG s_r_x_1000;   /* Sample rate * 1000 */
        LONG ns_sec;    /* nanoseconds per second */
};

/*------------------------Time Structure-------------------------------*/

struct	r_time	{
      	LONG sec;
	LONG ns;
};

/*-------------------------Call interface----------------------------------*/

struct detect_info {
	LONG	sam_sec;	/* Sample rate for this detector */
  	LONG 	drectyp;  /* Record type designator, used to select filter */
	LONG 	datapts;	/* Number of data points last decode */
	struct r_time startt;	/* Starting time of first data point */
	LONG	*indatar;	/* ptr to data array */
	LONG	lbksize;	/* Space reserved for lookback */
	LONG	*lbkarr;	/* ptr to lookback storage array */
	struct con_sto *incontd;   /* This detector's continuity structure */
	const char *detname;	/* Name of this event detector */
	BOOL	(*filterc)();	/* Pointer to the filter function (or NULL)*/
        double  *coefPtr;       /* Pointer to array of filter coefficients */
        LONG    nCoef;          /* Number of filter coefficients */
};

/*----------------------Auxillary onset information------------------------*/

struct s_onset {
	BOOL	new_onset;	/* Flag TRUE if new onset here */
  /*	TEXT	o_dalgo;  */	/* Detection algorithm 'A' or 'B' */
	UBYTE	o_snr[5];	/* SNR onset quality estimate values */
	
	ULONG	o_secs;
	ULONG	o_ns;
	ULONG	o_amps;		/* Amplitude of signal */
	ULONG	o_pl;		/* Integer part of period */
	ULONG	o_pr;		/* Next two decimals of period */
	ULONG	o_large;	/* twosd, background noise estimate */
        double  o_maxamp;       /* Amplitude in st dev */
        double  o_freq;         /* Frequency, in Hz */
};


/*----------------Macros for convenience--------------------------*/

#define	filhi_set(detector,value) (detector)->incontd->filhi = (value)
#define	fillo_set(detector,value) (detector)->incontd->fillo = (value)
#define iwin_set(detector,value) (detector)->incontd->iwin = (value)
#define	nhits_set(detector,value) (detector)->incontd->n_hits = (value)

#define	xth1_set(detector,value) (detector)->incontd->xth1 = (value)
#define	xth2_set(detector,value) (detector)->incontd->xth2 = (value)
#define	xth3_set(detector,value) (detector)->incontd->xth3 = (value)
#define	xthx_set(detector,value) (detector)->incontd->xthx = (value)

#define	deftc_set(detector,value) (detector)->incontd->def_tc = (value)
#define	wait_set(detector,value) (detector)->incontd->wait_blk = (value)
#define	avg_set(detector,value) (detector)->incontd->val_avg = (value)

#define setallparams(cs,fhi,flo,iw,nht,x1,x2,x3,xx,tc,wa,av) \
 { filhi_set(cs,fhi); fillo_set(cs,flo); iwin_set(cs,iw); nhits_set(cs,nht); \
   xth1_set(cs,x1); xth2_set(cs,x2); xth3_set(cs,x3); xthx_set(cs,xx); \
   deftc_set(cs,tc); wait_set(cs,wa); avg_set(cs,av); }

#ifdef __cplusplus
}
#endif

#endif //  _LIGOCONT_H
